using System;
using System.Web.UI;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.Web;
using System.Security.Permissions;
using System.Text;

namespace SharpPieces.Web.JQuery
{

    /// <summary>
    /// Represents the base class for SharpPieces JQuery extension controls.
    /// </summary>
    [DefaultProperty("TargetControlID")]
    public abstract class BaseJQueryExtension : BaseJQueryControl
    {

        /// <summary>
        /// Resolves the target control.
        /// </summary>
        protected virtual Control ResolveTargetControl()
        {
            return this.ResolveTargetControl<Control>();
        }

        /// <summary>
        /// Resolves the target control.
        /// </summary>
        /// <param name="assignableTo">The assignable to type; it can be null.</param>
        protected virtual T ResolveTargetControl<T>()
            where T : Control
        {
            Control targetControl = this.NamingContainer.FindControl(this.TargetControlID);
            if (null == targetControl)
            {
                throw new ArgumentException("Invalid TargetControlID.");
            }
            if ((null != typeof(T)) && !typeof(T).IsAssignableFrom(targetControl.GetType()))
            {
                throw new ArgumentException(string.Format("Invalid TargetControlID. The target type should be {0} or above.", typeof(T).Name));
            }
            return targetControl as T;
        }

        /// <summary>
        /// Converts a list of options to JavaScript representation.
        /// </summary>
        /// <param name="list">The options list.</param>
        internal static string OptionListToJS(IDictionary<string, string> list)
        {            
            if ((null != list)&& (0 < list.Count))
            {
                StringBuilder sb = new StringBuilder("{");
                foreach (KeyValuePair<string, string> option in list)
                {
                    sb.Append(string.Concat(option.Key, ":", option.Value, ","));
                }
                return sb.Replace(",", "}", sb.Length - 1, 1).ToString();
            }
            return "null";
        }


        // Properties

        ///// <summary>
        ///// Gets the client style references.
        ///// </summary>
        ///// <value>The client script styles.</value>
        //protected abstract IEnumerable<string> ClientStyleReferences { get; }

        /// <summary>
        /// Gets or sets the target control ID.
        /// </summary>
        /// <value>The target control ID.</value>
        [Description("The target control ID."), Category("Behavior"), DefaultValue("")]
        [Themeable(false), IDReferenceProperty]
        public virtual string TargetControlID
        {
            get { return this.ViewState["TargetControlID"] as string ?? string.Empty; }
            set { this.ViewState["TargetControlID"] = value; }
        }

        ///// <summary>
        ///// Gets or sets a value indicating whether this control uses custom styles or not.
        ///// Using custom styles the CSS and images need to be loaded manually.
        ///// </summary>
        ///// <value><c>true</c> if this control uses custom styles; otherwise, <c>false</c>.</value>
        //[Browsable(false)]
        //public virtual bool UseCustomStyles
        //{
        //    get { return (null != this.ViewState["UseCustomStyles"]) ? (bool)this.ViewState["UseCustomStyles"] : false; }
        //    set { this.ViewState["UseCustomStyles"] = value; }
        //}
    }

    /// <summary>
    /// Provides a type converter that retrieves a list of control IDs in the current 
    /// container, of the specified type.
    /// </summary>
    /// <typeparam name="T">The control type.</typeparam>
    public class GenericControlConverter<T> : ControlIDConverter
    {

        // Methods

        /// <summary>
        /// Returns a value indicating whether the control ID of the specified control is added to the <see cref="T:System.ComponentModel.TypeConverter.StandardValuesCollection"></see> that is returned by the <see cref="M:System.Web.UI.WebControls.ControlIDConverter.GetStandardValues(System.ComponentModel.ITypeDescriptorContext)"></see> method.
        /// </summary>
        /// <param name="control">The control instance to test for inclusion in the <see cref="T:System.ComponentModel.TypeConverter.StandardValuesCollection"></see>.</param>
        /// <returns>true in all cases.</returns>
        protected override bool FilterControl(Control control)
        {
            return typeof(T).IsInstanceOfType(control);
        }

    }

}



 
